@@ -13,6 +13,7 @@ class JobSettings(TypedDict):
 	job_id: uuid.UUID
 	message_id: int
 	chat_id: int
+	user_id: int
 	placeholder_message_id: int
 	local_media_path: str
 	local_compressed_media_path: str
@@ -42,10 +43,14 @@ class JobSettings(TypedDict):
 	chat_type: ChatType
 	account_admins: str
 	job_postponed_until: int
+	message_leftover: str
+	replay: bool
+	short_text: bool
 
 class AbstractJob(ABC):
 	job_id: uuid.UUID = None
 	message_id: int = 0
+	user_id: int = 0
 	chat_id: int = 0
 	placeholder_message_id: int = 0
 	local_media_path: str = ""
@@ -76,6 +81,9 @@ class AbstractJob(ABC):
 	chat_type: ChatType = None
 	account_admins: str = None
 	job_postponed_until: int = -1
+	message_leftover: str = ""
+	replay: bool = False
+	short_text: bool = False
 
 	def __init__(self, **kwargs: Unpack[JobSettings]) -> None:
 		if kwargs:
