@@ -5,7 +5,9 @@ from typing import Optional
 import multiprocessing
 from queue import Empty
 
-from warp_beacon.scraper.exceptions import NotFound, UnknownError, TimeOut, Unavailable, FileTooBig, YotubeLiveError, YotubeAgeRestrictedError, IGRateLimitOccurred, CaptchaIssue, AllAccountsFailed
+import logging
+
+from warp_beacon.scraper.exceptions import NotFound, UnknownError, TimeOut, Unavailable, FileTooBig, YoutubeLiveError, YotubeAgeRestrictedError, IGRateLimitOccurred, CaptchaIssue, AllAccountsFailed
 from warp_beacon.mediainfo.video import VideoInfo
 from warp_beacon.mediainfo.audio import AudioInfo
 from warp_beacon.mediainfo.silencer import Silencer
@@ -16,10 +18,12 @@ from warp_beacon.jobs.download_job import DownloadJob
 from warp_beacon.jobs.upload_job import UploadJob
 from warp_beacon.jobs.types import JobType
 from warp_beacon.scraper.account_selector import AccountSelector
-
-import logging
+from warp_beacon.storage.mongo import DBClient
+from warp_beacon.scraper.fail_handler import FailHandler
+from warp_beacon.scraper.link_resolver import LinkResolver
 
 ACC_FILE = os.environ.get("SERVICE_ACCOUNTS_FILE", default="/var/warp_beacon/accounts.json")
+PROXY_FILE = os.environ.get("PROXY_FILE", default="/var/warp_beacon/proxies.json")
 
 class AsyncDownloader(object):
 	__JOE_BIDEN_WAKEUP = None
@@ -36,7 +40,7 @@ class AsyncDownloader(object):
 	def __init__(self, uploader: AsyncUploader, workers_count: int) -> None:
 		self.manager = multiprocessing.Manager()
 		self.allow_loop = self.manager.Value('i', 1)
-		self.acc_selector = AccountSelector(self.manager, ACC_FILE)
+		self.acc_selector = AccountSelector(self.manager, ACC_FILE, PROXY_FILE)
 		self.uploader = uploader
 		self.workers_count = workers_count
 
@@ -83,22 +87,29 @@ class AsyncDownloader(object):
 
 	def do_work(self, selector: AccountSelector) -> None:
 		logging.info("download worker started")
+		# pymongo is not fork-safe so new connect to DB required
+		fail_handler = FailHandler(DBClient())
 		while self.allow_loop.value == 1:
 			try:
 				job = None
 				try:
 					job = self.job_queue.get()
 					if job is self.__JOE_BIDEN_WAKEUP:
-						continue
+						break
 					actor = None
 					try:
 						items = []
 						if job.job_origin is Origin.UNKNOWN:
 							logging.warning("Unknown task origin! Skipping.")
 							continue
+						if LinkResolver.resolve_job(job):
+							self.uploader.queue_task(job.to_upload_job(
+								replay=True
+							))
+							continue
 						if not job.in_process:
 							if job.job_postponed_until > 0:
-								if job.job_postponed_until - time.time() > 0:
+								if (job.job_postponed_until - time.time()) > 0:
 									logging.warning("Job '%s' is postponed, rescheduling", job.url)
 									time.sleep(2)
 									self.job_queue.put(job)
@@ -107,16 +118,16 @@ class AsyncDownloader(object):
 							self.acc_selector.set_module(job.job_origin)
 							if job.job_origin is Origin.INSTAGRAM:
 								from warp_beacon.scraper.instagram.instagram import InstagramScraper
-								actor = InstagramScraper(selector.get_current())
+								actor = InstagramScraper(selector.get_current(), selector.get_account_proxy())
 							elif job.job_origin is Origin.YT_SHORTS:
 								from warp_beacon.scraper.youtube.shorts import YoutubeShortsScraper
-								actor = YoutubeShortsScraper(selector.get_current())
+								actor = YoutubeShortsScraper(selector.get_current(), selector.get_account_proxy())
 							elif job.job_origin is Origin.YT_MUSIC:
 								from warp_beacon.scraper.youtube.music import YoutubeMusicScraper
-								actor = YoutubeMusicScraper(selector.get_current())
+								actor = YoutubeMusicScraper(selector.get_current(), selector.get_account_proxy())
 							elif job.job_origin is Origin.YOUTUBE:
 								from warp_beacon.scraper.youtube.youtube import YoutubeScraper
-								actor = YoutubeScraper(selector.get_current())
+								actor = YoutubeScraper(selector.get_current(), selector.get_account_proxy())
 							actor.send_message_to_admin_func = self.send_message_to_admin
 							actor.auth_event = self.auth_event
 							while True:
@@ -127,7 +138,7 @@ class AsyncDownloader(object):
 										logging.info("done")
 									else:
 										logging.info("Downloading URL '%s'", job.url)
-										items = actor.download(job.url)
+										items = actor.download(job)
 									break
 								except NotFound as e:
 									logging.warning("Not found error occurred!")
@@ -169,7 +180,7 @@ class AsyncDownloader(object):
 									logging.warning("Telegram limits exceeded :(")
 									logging.exception(e)
 									self.send_message_to_admin(
-										f"Task <code>{job.job_id}</code> failed. URL: '{job.url}'. Reason: '<b>FileTooBi</b>'."
+										f"Task <code>{job.job_id}</code> failed. URL: '{job.url}'. Reason: '<b>FileTooBig</b>'."
 									)
 									self.uploader.queue_task(job.to_upload_job(
 										job_failed=True,
@@ -192,7 +203,7 @@ class AsyncDownloader(object):
 									self.try_next_account(selector, job, report_error="captcha")
 									self.job_queue.put(job)
 									break
-								except YotubeLiveError as e:
+								except YoutubeLiveError as e:
 									logging.warning("Youtube Live videos are not supported. Skipping.")
 									logging.exception(e)
 									self.uploader.queue_task(job.to_upload_job(
@@ -222,9 +233,15 @@ class AsyncDownloader(object):
 										job_failed_msg="All bot accounts failed to download content. Bot administrator noticed about the issue.")
 									)
 									if e.job.job_origin == Origin.INSTAGRAM:
-										self.try_next_account(selector, job, report_error="captcha")
-										e.job.job_timeout = time.time() + 300
-										self.job_queue.put(e.job)
+										logging.info("Handling captcha postpone")
+										self.uploader.queue_task(job.to_upload_job(
+											job_warning=True,
+											job_warning_msg="Bot is experiencing issues, video delivery may be delayed.")
+										)
+										#self.try_next_account(selector, job, report_error="captcha")
+										#e.job.job_postponed_until = time.time() + 300
+										#self.job_queue.put(e.job)
+										fail_handler.store_failed_job(e.job)
 									break
 								except (UnknownError, Exception) as e:
 									logging.warning("UnknownError occurred!")
@@ -260,6 +277,9 @@ class AsyncDownloader(object):
 									break
 
 							if items:
+								# success
+								for job in fail_handler.get_failed_jobs():
+									self.queue_task(job)
 								for item in items:
 									media_info = {"filesize": 0}
 									if item["media_type"] == JobType.VIDEO:
