@@ -11,7 +11,7 @@ class ScraperError(Exception):
 				body=getattr(self, "error_type", vars(self)),
 			)
 		super().__init__(self.message, *args, **kwargs)
-		if hasattr(self, "response") and self.response: 
+		if hasattr(self, "response") and self.response:
 			self.code = self.response.status_code
 
 class TimeOut(ScraperError):
@@ -26,7 +26,7 @@ class NotFound(ScraperError):
 class Unavailable(ScraperError):
 	pass
 
-class YotubeLiveError(ScraperError):
+class YoutubeLiveError(ScraperError):
 	pass
 
 class YotubeAgeRestrictedError(ScraperError):
@@ -54,6 +54,8 @@ def extract_exception_message(e: Exception) -> str:
 		msg = e.message
 	elif hasattr(e, "reason"):
 		msg = e.reason
+	elif hasattr(e, "msg"):
+		msg = e.msg
 	elif hasattr(e, "args"):
 		msg = str(e.args)
 		
