@@ -14,14 +14,16 @@ import urllib3
 from urllib.parse import urljoin, urlparse
 
 from instagrapi.mixins.story import Story
-#from instagrapi.types import Media
+from instagrapi.types import Media
 from instagrapi import Client
 from instagrapi.mixins.challenge import ChallengeChoice
-from instagrapi.exceptions import LoginRequired, PleaseWaitFewMinutes, MediaNotFound, ClientNotFoundError, UserNotFound, ChallengeRequired, ChallengeSelfieCaptcha, UnknownError as IGUnknownError
+from instagrapi.exceptions import LoginRequired, PleaseWaitFewMinutes, MediaNotFound, ClientNotFoundError, UserNotFound, ChallengeRequired, \
+	ChallengeSelfieCaptcha, ChallengeUnknownStep, UnknownError as IGUnknownError
 
 from warp_beacon.scraper.exceptions import NotFound, UnknownError, TimeOut, IGRateLimitOccurred, CaptchaIssue, extract_exception_message
 from warp_beacon.scraper.abstract import ScraperAbstract
 from warp_beacon.jobs.types import JobType
+from warp_beacon.jobs.download_job import DownloadJob
 from warp_beacon.telegram.utils import Utils
 
 import logging
@@ -31,12 +33,18 @@ INST_SESSION_FILE_TPL = "/var/warp_beacon/inst_session_account_%d.json"
 class InstagramScraper(ScraperAbstract):
 	cl = None
 	inst_session_file = ""
+	timeline_cursor = {}
 
-	def __init__(self, account: tuple) -> None:
-		super().__init__(account)
+	def __init__(self, account: tuple, proxy: dict=None) -> None:
+		super().__init__(account, proxy)
 		#
 		self.inst_session_file = INST_SESSION_FILE_TPL % self.account_index
 		self.cl = Client()
+		if self.proxy:
+			proxy_dsn = self.proxy.get("dsn", "")
+			if proxy_dsn:
+				self.cl.set_proxy(proxy_dsn)
+				logging.info("Using proxy DSN '%s'", proxy_dsn)
 		#self.cl.logger.setLevel(logging.DEBUG)
 		self.setup_device()
 		self.cl.challenge_code_handler = self.challenge_code_handler
@@ -64,9 +72,11 @@ class InstagramScraper(ScraperAbstract):
 		})
 
 	def safe_write_session(self) -> None:
+		cl_settings = self.cl.get_settings()
+		cl_settings["warp_timeline_cursor"] = self.timeline_cursor
 		tmp_fname = "%s~" % self.inst_session_file
 		with open(tmp_fname, 'w+', encoding="utf-8") as f:
-			f.write(json.dumps(self.cl.get_settings()))
+			f.write(json.dumps(cl_settings))
 		if os.path.exists(self.inst_session_file):
 			os.unlink(self.inst_session_file)
 		os.rename(tmp_fname, self.inst_session_file)
@@ -74,9 +84,14 @@ class InstagramScraper(ScraperAbstract):
 	def load_session(self) -> None:
 		if os.path.exists(self.inst_session_file):
 			logging.info("Loading existing session file '%s'", self.inst_session_file)
-			self.cl.load_settings(self.inst_session_file)
+			with open(self.inst_session_file, 'r', encoding="utf-8") as f:
+				js = json.loads(f.read())
+				if "warp_timeline_cursor" in js:
+					self.timeline_cursor = js.get("warp_timeline_cursor", {})
+					del js["warp_timeline_cursor"]
+				self.cl.set_settings(js)
 		else:
-			self.login()
+			self.download_hndlr(self.login)
 
 	def login(self) -> None:
 		username = self.account["login"]
@@ -86,12 +101,10 @@ class InstagramScraper(ScraperAbstract):
 		self.safe_write_session()
 
 	def validate_session(self) -> None:
+		from warp_beacon.scheduler.instagram_human import InstagramHuman
 		self.load_session()
-		self._download_hndlr(self.cl.get_timeline_feed)
-		self._download_hndlr(self.cl.get_reels_tray_feed, "cold_start")
-		self._download_hndlr(self.cl.direct_active_presence)
-		self._download_hndlr(self.cl.reels)
-		#self._download_hndlr(self.cl.notification_like_and_comment_on_photo_user_tagged, "everyone")
+		inst_human = InstagramHuman(self)
+		inst_human.simulate_activity()
 		self.safe_write_session()
 
 	def scrap(self, url: str) -> tuple[str]:
@@ -131,7 +144,7 @@ class InstagramScraper(ScraperAbstract):
 		logging.info("media_id is '%s'", media_id)
 		return media_id
 	
-	def _download_hndlr(self, func: Callable, *args: tuple[str], **kwargs: dict[str]) -> Union[str, dict]:
+	def download_hndlr(self, func: Callable, *args: tuple[str], **kwargs: dict[str]) -> Union[str, dict]:
 		ret_val = {}
 		max_retries = int(os.environ.get("IG_MAX_RETRIES", default=5))
 		retries = 0
@@ -139,7 +152,7 @@ class InstagramScraper(ScraperAbstract):
 			try:
 				ret_val = func(*args, **kwargs)
 				break
-			except (ChallengeRequired, ChallengeSelfieCaptcha) as e:
+			except (ChallengeRequired, ChallengeSelfieCaptcha, ChallengeUnknownStep) as e:
 				logging.warning("Instagram wants Challange!")
 				logging.exception(e)
 				raise CaptchaIssue("a captcha issue arose")
@@ -172,19 +185,20 @@ class InstagramScraper(ScraperAbstract):
 
 		return ret_val
 
-	def download_video(self, url: str, media_info: dict) -> dict:
+	def download_video(self, url: str, media_info: Media) -> dict:
 		self.cl.request_timeout = int(os.environ.get("IG_REQUEST_TIMEOUT", default=60))
-		path = self._download_hndlr(self.cl.video_download_by_url, url, folder='/tmp')
-		return {"local_media_path": str(path), "media_type": JobType.VIDEO, "media_info": {"duration": round(media_info.video_duration)}}
+		path = self.download_hndlr(self.cl.video_download_by_url, url, folder='/tmp')
+		return {"local_media_path": str(path), "canonical_name": self.extract_canonical_name(media_info), \
+			"media_type": JobType.VIDEO, "media_info": {"duration": round(media_info.video_duration)}}
 
-	def download_photo(self, url: str) -> dict:
-		path = str(self._download_hndlr(self.cl.photo_download_by_url, url, folder='/tmp'))
+	def download_photo(self, url: str, media_info: Media) -> dict:
+		path = str(self.download_hndlr(self.cl.photo_download_by_url, url, folder='/tmp'))
 		path_lowered = path.lower()
 		if ".webp" in path_lowered:
 			path = InstagramScraper.convert_webp_to_png(path)
 		if ".heic" in path_lowered:
 			path = InstagramScraper.convert_heic_to_png(path)
-		return {"local_media_path": path, "media_type": JobType.IMAGE}
+		return {"local_media_path": path, "canonical_name": self.extract_canonical_name(media_info), "media_type": JobType.IMAGE}
 	
 	def download_story(self, story_info: Story) -> dict:
 		path, media_type, media_info = "", JobType.UNKNOWN, {}
@@ -197,7 +211,7 @@ class InstagramScraper(ScraperAbstract):
 		logging.info("Effective story id is '%s'", effective_story_id)
 		effective_url = "https://www.instagram.com/stories/%s/%s/" % (story_info.user.username, effective_story_id)
 		if story_info.media_type == 1: # photo
-			path = str(self._download_hndlr(self.cl.story_download_by_url, url=story_info.thumbnail_url, folder='/tmp'))
+			path = str(self.download_hndlr(self.cl.story_download_by_url, url=story_info.thumbnail_url, folder='/tmp'))
 			path_lowered = path.lower()
 			if ".webp" in path_lowered:
 				path = InstagramScraper.convert_webp_to_png(path)
@@ -205,7 +219,7 @@ class InstagramScraper(ScraperAbstract):
 				path = InstagramScraper.convert_heic_to_png(path)
 			media_type = JobType.IMAGE
 		elif story_info.media_type == 2: # video
-			path = str(self._download_hndlr(self.cl.story_download_by_url, url=story_info.video_url, folder='/tmp'))
+			path = str(self.download_hndlr(self.cl.story_download_by_url, url=story_info.video_url, folder='/tmp'))
 			media_type = JobType.VIDEO
 			media_info["duration"] = story_info.video_duration
 
@@ -221,32 +235,45 @@ class InstagramScraper(ScraperAbstract):
 
 		return {"media_type": JobType.COLLECTION, "save_items": True, "items": chunks}
 
-	def download_album(self, media_info: dict) -> dict:
+	def download_album(self, media_info: Media) -> dict:
 		chunks = []
 		for media_chunk in Utils.chunker(media_info.resources, 10):
 			chunk = []
 			for media in media_chunk:
-				_media_info = self._download_hndlr(self.cl.media_info, media.pk)
+				_media_info = self.download_hndlr(self.cl.media_info, media.pk)
 				if media.media_type == 1: # photo
-					chunk.append(self.download_photo(url=_media_info.thumbnail_url))
+					chunk.append(self.download_photo(url=_media_info.thumbnail_url, media_info=_media_info))
 				elif media.media_type == 2: # video
 					chunk.append(self.download_video(url=_media_info.video_url, media_info=_media_info))
 			chunks.append(chunk)
 
 		return {"media_type": JobType.COLLECTION, "items": chunks}
 
-	def download(self, url: str) -> Optional[list[dict]]:
+	def extract_canonical_name(self, media: Media) -> str:
+		ret = ""
+		try:
+			if media.title:
+				ret = media.title
+			if media.caption_text:
+				ret += "\n" + media.caption_text
+		except Exception as e:
+			logging.warning("Failed to extract canonical media name!")
+			logging.exception(e)
+
+		return ret
+
+	def download(self, job: DownloadJob) -> Optional[list[dict]]:
 		res = []
 		while True:
 			try:
-				scrap_type, media_id = self.scrap(url)
+				scrap_type, media_id = self.scrap(job.url)
 				if scrap_type == "media":
-					media_info = self._download_hndlr(self.cl.media_info, media_id)
+					media_info = self.download_hndlr(self.cl.media_info, media_id)
 					logging.info("media_type is '%d', product_type is '%s'", media_info.media_type, media_info.product_type)
 					if media_info.media_type == 2 and media_info.product_type == "clips": # Reels
 						res.append(self.download_video(url=media_info.video_url, media_info=media_info))
 					elif media_info.media_type == 1: # Photo
-						res.append(self.download_photo(url=media_info.thumbnail_url))
+						res.append(self.download_photo(url=media_info.thumbnail_url, media_info=media_info))
 					elif media_info.media_type == 8: # Album
 						res.append(self.download_album(media_info=media_info))
 				elif scrap_type == "story":
