@@ -1,4 +1,5 @@
-import os, io
+import os
+import io
 import pathlib
 import time
 import math
@@ -6,24 +7,29 @@ import socket
 import ssl
 #from abc import abstractmethod
 from typing import Callable, Union, Optional
+import logging
 import json
 import urllib
-import requests
 import http.client
+import pytubefix.exceptions
+import requests
 from PIL import Image
+import numpy as np
+from urllib.parse import urlparse, parse_qs
 
-from warp_beacon.scraper.abstract import ScraperAbstract
-from warp_beacon.mediainfo.abstract import MediaInfoAbstract
-from warp_beacon.scraper.exceptions import TimeOut, Unavailable, extract_exception_message
-
+import pytubefix
 from pytubefix import YouTube
 from pytubefix.innertube import _default_clients
 from pytubefix.streams import Stream
 from pytubefix.innertube import InnerTube, _client_id, _client_secret
-from pytubefix.exceptions import VideoUnavailable, VideoPrivate, MaxRetriesExceeded
+#from pytubefix.exceptions import VideoUnavailable, VideoPrivate, MaxRetriesExceeded
 from pytubefix import request
+import yt_dlp
 
-import logging
+from warp_beacon.jobs.download_job import DownloadJob
+from warp_beacon.scraper.abstract import ScraperAbstract
+#from warp_beacon.mediainfo.abstract import MediaInfoAbstract
+from warp_beacon.scraper.exceptions import TimeOut, Unavailable, extract_exception_message
 
 def patched_fetch_bearer_token(self) -> None:
 	"""Fetch an OAuth token."""
@@ -79,11 +85,11 @@ class YoutubeAbstract(ScraperAbstract):
 	DOWNLOAD_DIR = "/tmp"
 	YT_SESSION_FILE = '/var/warp_beacon/yt_session_%d.json'
 
-	def __init__(self, account: tuple) -> None:
-		super().__init__(account)
+	#def __init__(self, account: tuple, proxy: dict=None) -> None:
+	#	super().__init__(account, proxy)
 
-	def __del__(self) -> None:
-		pass
+	#def __del__(self) -> None:
+	#	pass
 
 	def rename_local_file(self, filename: str) -> str:
 		if not os.path.exists(filename):
@@ -98,13 +104,24 @@ class YoutubeAbstract(ScraperAbstract):
 		os.rename(filename, new_filepath)
 
 		return new_filepath
+	
+	def get_video_id(self, url: str) -> Optional[str]:
+		parsed_url = urlparse(url)
+		query = parse_qs(parsed_url.query)
+		return query.get('v', [None])[0]
 
 	def remove_tmp_files(self) -> None:
 		for i in os.listdir(self.DOWNLOAD_DIR):
 			if "yt_download_" in i:
 				os.unlink("%s/%s" % (self.DOWNLOAD_DIR, i))
 
-	def calculate_size_with_padding(self, image: Image, aspect_ratio_width: int, aspect_ratio_height: int, target_size: tuple=(320, 320), background_color: tuple=(0, 0, 0)) -> Image:
+	def calculate_size_with_padding(self,
+		image: Image,
+		aspect_ratio_width: int,
+		aspect_ratio_height: int,
+		target_size: tuple=(320, 320),
+		background_color: tuple=(0, 0, 0)
+	) -> Image:
 		aspect_ratio = aspect_ratio_width / aspect_ratio_height
 		target_width, target_height = target_size
 
@@ -127,12 +144,32 @@ class YoutubeAbstract(ScraperAbstract):
 
 		return new_image
 
+	def resize_aspect_ratio(self, image: Image, base_width: int = 320) -> Image:
+		wpercent = (base_width / float(image.size[0]))
+		hsize = int((float(image.size[1]) * float(wpercent)))
+		img = image.resize((base_width, hsize), Image.Resampling.LANCZOS)
+
+		return img
+
 	def aspect_ratio(self, size: tuple) -> tuple:
 		gcd = math.gcd(size[0], size[1])
 		return size[0] // gcd, size[1] // gcd
+	
+	def crop_black_edges_pil(self, img: Image, threshold: int = 35) -> Image:
+		y_nonzero, x_nonzero, _ = np.nonzero(np.array(img) > threshold)
+		return img.crop((np.min(x_nonzero), np.min(y_nonzero), np.max(x_nonzero), np.max(y_nonzero)))
 
-	def download_thumbnail(self, video_id: str, timeout: int) -> Optional[io.BytesIO]:
-		for i in (#"https://img.youtube.com/vi/{VIDEO_ID}/maxresdefault.jpg",
+	def crop_center(self, image: Image, width: int, height: int) -> Image:
+		img_width, img_height = image.size
+		left = (img_width - width) // 2
+		top = (img_height - height) // 2
+		right = left + width
+		bottom = top + height
+
+		return image.crop((left, top, right, bottom))
+
+	def download_thumbnail(self, video_id: str, timeout: int, crop_center: dict = None) -> Optional[io.BytesIO]:
+		for i in ("https://img.youtube.com/vi/{VIDEO_ID}/maxresdefault.jpg",
 				"https://img.youtube.com/vi/{VIDEO_ID}/hqdefault.jpg",
 				"https://img.youtube.com/vi/{VIDEO_ID}/sddefault.jpg"):
 			try:
@@ -142,15 +179,14 @@ class YoutubeAbstract(ScraperAbstract):
 					if response.status_code == 200:
 						image = Image.open(io.BytesIO(response.content))
 						ratio = self.aspect_ratio(image.size)
+						image = self.crop_black_edges_pil(image)
+						if crop_center:
+							image = self.crop_center(image, width=crop_center["width"], height=crop_center["height"])
 						logging.info("thumb ratio: '%s'", ratio)
-						new_image = None
-						if ratio[1] > 4:
-							new_image = MediaInfoAbstract.shrink_image_to_fit(image)
-						else:
-							new_image = self.calculate_size_with_padding(image, ratio[0], ratio[1])
+						new_image = self.resize_aspect_ratio(image)
 						logging.info("thumb size: '%s'", new_image.size)
 						io_buf = io.BytesIO()
-						new_image.save(io_buf, format='JPEG', subsampling=0, quality=100, progressive=True, optimize=False)
+						new_image.save(io_buf, format='JPEG', subsampling=0, quality=95, progressive=True, optimize=False)
 						logging.info("thumb size: %d kb", io_buf.getbuffer().nbytes / 1024)
 						io_buf.seek(0)
 						return io_buf
@@ -160,8 +196,8 @@ class YoutubeAbstract(ScraperAbstract):
 
 		return None
 
-	def _download_hndlr(self, func: Callable, *args: tuple[str], **kwargs: dict[str]) -> Union[str, dict, io.BytesIO]:
-		ret_val = ''
+	def download_hndlr(self, func: Callable, *args: tuple[Union[str, int, dict, tuple, bool]], **kwargs: dict[Union[str, int, dict, tuple, bool]]) -> Optional[Union[list, dict, str, io.BytesIO]]:
+		ret_val = None
 		max_retries = int(os.environ.get("YT_MAX_RETRIES", default=self.YT_MAX_RETRIES_DEFAULT))
 		pause_secs = int(os.environ.get("YT_PAUSE_BEFORE_RETRY", default=self.YT_PAUSE_BEFORE_RETRY_DEFAULT))
 		timeout = int(os.environ.get("YT_TIMEOUT", default=self.YT_TIMEOUT_DEFAULT))
@@ -172,7 +208,7 @@ class YoutubeAbstract(ScraperAbstract):
 				kwargs["timeout"] = timeout
 				ret_val = func(*args, **kwargs)
 				break
-			except MaxRetriesExceeded:
+			except pytubefix.exceptions.MaxRetriesExceeded:
 				# do noting, not interested
 				pass
 			#except http.client.IncompleteRead as e:
@@ -183,10 +219,10 @@ class YoutubeAbstract(ScraperAbstract):
 					requests.RequestException,
 					urllib.error.URLError,
 					urllib.error.HTTPError) as e:
-				if hasattr(e, "code") and int(e.code) == 403:
+				if hasattr(e, "code") and (int(e.code) == 403 or int(e.code) == 400):
 					raise Unavailable(extract_exception_message(e))
-				logging.warning("Youtube read timeout! Retrying in %d seconds ...", pause_secs)
-				logging.info("Your `YT_MAX_RETRIES` values is %d", max_retries)
+				logging.warning("Youtube read timeout! Retrying in '%d' seconds ...", pause_secs)
+				logging.info("Your `YT_MAX_RETRIES` values is '%d'", max_retries)
 				logging.exception(extract_exception_message(e))
 				if max_retries <= retries:
 					self.remove_tmp_files()
@@ -194,7 +230,21 @@ class YoutubeAbstract(ScraperAbstract):
 				retries += 1
 				timeout += timeout_increment
 				time.sleep(pause_secs)
-			except (VideoUnavailable, VideoPrivate) as e:
+			except (pytubefix.exceptions.VideoUnavailable, pytubefix.exceptions.VideoPrivate) as e:
+				raise Unavailable(extract_exception_message(e))
+			except yt_dlp.utils.DownloadError as e:
+				raise Unavailable(extract_exception_message(e))
+			except yt_dlp.utils.GeoRestrictedError:
+				raise Unavailable(extract_exception_message(e))
+			except yt_dlp.utils.PostProcessingError as e:
+				raise Unavailable(extract_exception_message(e))
+			except yt_dlp.utils.ExtractorError as e:
+				raise Unavailable(extract_exception_message(e))
+			except yt_dlp.utils.MaxDownloadsReached as e:
+				raise Unavailable(extract_exception_message(e))
+			except yt_dlp.utils.UnavailableVideoError as e:
+				raise Unavailable(extract_exception_message(e))
+			except yt_dlp.utils.ThrottledDownload as e:
 				raise Unavailable(extract_exception_message(e))
 
 		return ret_val
@@ -203,16 +253,71 @@ class YoutubeAbstract(ScraperAbstract):
 		pass
 		#logging.info("bytes: %d, bytes remaining: %d", chunk, bytes_remaining)
 
-	def build_yt(self, url: str) -> YouTube:
-		InnerTube.send_message_to_admin_func = self.send_message_to_admin_func
-		InnerTube.auth_event = self.auth_event
-		InnerTube.wb_account = self.account
-		InnerTube.fetch_bearer_token = patched_fetch_bearer_token
+	def build_yt(self, url: str, session: bool = True) -> YouTube:
+		if session:
+			InnerTube.send_message_to_admin_func = self.send_message_to_admin_func
+			InnerTube.auth_event = self.auth_event
+			InnerTube.wb_account = self.account
+			InnerTube.fetch_bearer_token = patched_fetch_bearer_token
 		_default_clients["ANDROID"]["innertube_context"]["context"]["client"]["clientVersion"] = "19.08.35"
 		_default_clients["ANDROID_MUSIC"] = _default_clients["ANDROID"]
 		yt_opts = {"url": url, "on_progress_callback": self.yt_on_progress}
-		#yt_opts["client"] = "WEB"
-		yt_opts["use_oauth"] = True
-		yt_opts["allow_oauth_cache"] = True
-		yt_opts["token_file"] = self.YT_SESSION_FILE % self.account_index
+		if session:
+			yt_opts["client"] = "TV_EMBED"
+			yt_opts["use_oauth"] = True
+			yt_opts["allow_oauth_cache"] = True
+			yt_opts["token_file"] = self.YT_SESSION_FILE % self.account_index
+		if self.proxy:
+			proxy_dsn = self.proxy.get("dsn", "")
+			logging.info("Using proxy DSN '%s'", proxy_dsn)
+			if proxy_dsn:
+				if "https://" in proxy_dsn:
+					yt_opts["proxies"] = {"https": proxy_dsn}
+				elif "http://" in proxy_dsn:
+					yt_opts["proxies"] = {"http": proxy_dsn}
+				else:
+					logging.warning("Proxy DSN malformed!")
 		return YouTube(**yt_opts)
+	
+	def build_yt_dlp(self, timeout: int = 60) -> yt_dlp.YoutubeDL:
+		auth_data = {}
+		with open(self.YT_SESSION_FILE % self.account_index, 'r', encoding="utf-8") as f:
+			auth_data = json.loads(f.read())
+		time_name = str(time.time()).replace('.', '_')
+		ydl_opts = {
+			'socket_timeout': timeout,
+			'outtmpl': f'{self.DOWNLOAD_DIR}/yt_download_{time_name}.%(ext)s',
+			'format': 'bestvideo+bestaudio/best',
+			'merge_output_format': 'mp4',
+			'noplaylist': True,
+			'tv_auth': auth_data
+		}
+
+		if self.proxy:
+			proxy_dsn = self.proxy.get("dsn", "")
+			logging.info("Using proxy DSN '%s'", proxy_dsn)
+			if proxy_dsn:
+				ydl_opts["proxy"] = proxy_dsn
+
+		return yt_dlp.YoutubeDL(ydl_opts)
+	
+	def _download(self, _: str, __: int = 60) -> list:
+		raise NotImplementedError("You should to implement _download method")
+	
+	def _download_yt_dlp(self, _: str, __: int = 60) -> list:
+		raise NotImplementedError("You should to implement _download_yt_dlp method")
+
+	def download(self, job: DownloadJob) -> list:
+		ret = []
+		try:
+			ret = self.download_hndlr(self._download, job.url, session=True)
+		except (Unavailable, TimeOut):
+			logging.warning("Download failed, trying to download with yt_dlp")
+		
+		try:
+			ret = self.download_hndlr(self._download_yt_dlp, job.url)
+		except NotImplementedError:
+			logging.info("yt_dlp is not supported for this submodule yet")
+			raise Unavailable("Сontent unvailable")
+
+		return ret
