@@ -2,6 +2,8 @@ from warp_beacon.jobs.types import JobType
 from warp_beacon.scraper.youtube.abstract import YoutubeAbstract
 from warp_beacon.scraper.exceptions import NotFound
 
+from warp_beacon.mediainfo.video import VideoInfo
+
 import logging
 
 class YoutubeShortsScraper(YoutubeAbstract):
@@ -10,18 +12,15 @@ class YoutubeShortsScraper(YoutubeAbstract):
 	YT_TIMEOUT_DEFAULT = 2
 	YT_TIMEOUT_INCREMENT_DEFAULT = 60
 
-	def _download(self, url: str, timeout: int = 0) -> list:
+	def _download(self, url: str, session: bool = True, timeout: int = 0) -> list:
 		res = []
 		thumbnail = None
-		yt = self.build_yt(url)
+		yt = self.build_yt(url, session=session)
 		stream = yt.streams.get_highest_resolution()
 
 		if not stream:
 			raise NotFound("No suitable video stream found")
 
-		if yt:
-			thumbnail = self._download_hndlr(self.download_thumbnail, yt.video_id)
-
 		local_file = stream.download(
 			output_path=self.DOWNLOAD_DIR,
 			max_retries=0,
@@ -29,16 +28,18 @@ class YoutubeShortsScraper(YoutubeAbstract):
 			skip_existing=False,
 			filename_prefix="yt_download_"
 		)
+
+		local_file = self.rename_local_file(local_file)
+		vinfo = VideoInfo(local_file)
+		thumbnail = self.download_hndlr(self.download_thumbnail, video_id=yt.video_id, crop_center=vinfo.get_demensions())
+
 		logging.debug("Temp filename: '%s'", local_file)
 		res.append({
-			"local_media_path": self.rename_local_file(local_file),
+			"local_media_path": local_file,
 			"performer": yt.author,
 			"thumb": thumbnail,
 			"canonical_name": stream.title,
 			"media_type": JobType.VIDEO
 		})
 
-		return res
-
-	def download(self, url: str) -> list:
-		return self._download_hndlr(self._download, url)
\ No newline at end of file
+		return res
\ No newline at end of file
