@@ -4,7 +4,7 @@ from enum import Enum
 
 from urllib.parse import urlparse, parse_qs
 
-from pymongo import MongoClient
+from warp_beacon.storage.mongo import DBClient
 
 import logging
 
@@ -19,20 +19,14 @@ VIDEO_STORAGE_DIR = os.environ.get("VIDEO_STORAGE_DIR", default="/var/warp_beaco
 class Storage(object):
 	client = None
 	db = None
-	def __init__(self) -> None:
+	def __init__(self, client: DBClient) -> None:
 		if not os.path.isdir(VIDEO_STORAGE_DIR):
 			os.mkdir(VIDEO_STORAGE_DIR)
-
-		self.client = MongoClient(
-			host=os.environ.get("MONGODB_HOST", default='127.0.0.1'), 
-			port=int(os.environ.get("MONGODB_PORT", default=27017)),
-			username=os.environ.get("MONGODB_USER", default='root'),
-			password=os.environ.get("MONGODB_PASSWORD", default="changeme"))
-		self.db = self.client.media.media
+		self.client = client
+		self.db = self.client.client.media.media
 
 	def __del__(self) -> None:
-		if self.client:
-			self.client.close()
+		self.client.close()
 
 	@staticmethod
 	def compute_uniq(url: str) -> str:
@@ -58,12 +52,15 @@ class Storage(object):
 		path = urlparse(url).path.strip('/')
 		return path
 	
-	def db_find(self, uniq_id: str) -> list[dict]:
+	def db_find(self, uniq_id: str, origin: str = "") -> list[dict]:
 		document = None
 		ret = []
 		try:
 			logging.debug("uniq_id to search is '%s'", uniq_id)
-			cursor = self.db.find({"uniq_id": uniq_id})
+			find_opts = {"uniq_id": uniq_id}
+			if origin:
+				find_opts["origin"] = origin
+			cursor = self.db.find(find_opts)
 			for document in cursor:
 				ret.append(
 				{
@@ -119,7 +116,4 @@ class Storage(object):
 		except Exception as e:
 			logging.error("Error occurred while trying to read from the database!")
 			logging.exception(e)
-		return ret
-
-
-
+		return ret
\ No newline at end of file
