@@ -1,6 +1,8 @@
 import os
 import signal
 
+import logging
+
 import html
 
 import uvloop
@@ -15,17 +17,18 @@ import warp_beacon
 from warp_beacon.__version__ import __version__
 from warp_beacon.telegram.handlers import Handlers
 from warp_beacon.telegram.placeholder_message import PlaceholderMessage
+from warp_beacon.storage.mongo import DBClient
 from warp_beacon.storage import Storage
 from warp_beacon.uploader import AsyncUploader
 from warp_beacon.jobs.upload_job import UploadJob
 from warp_beacon.jobs.types import JobType
+from warp_beacon.jobs import Origin
 from warp_beacon.telegram.utils import Utils
+from warp_beacon.telegram.caption_shortener import CaptionShortner
 from warp_beacon.scheduler.scheduler import IGScheduler
 
-import logging
-
 class Bot(object):
-	storage = Storage()
+	storage = None
 	uploader = None
 	downloader = None
 	allow_loop = True
@@ -40,6 +43,11 @@ class Bot(object):
 			format="%(asctime)s - %(name)s - %(levelname)s - %(message)s", level=logging.INFO
 		)
 
+		logging.getLogger("pyrogram").setLevel(logging.ERROR)
+
+		db_connect = DBClient()
+		self.storage = Storage(db_connect)
+
 		logging.info("Starting Warp Beacon version '%s' ...", __version__)
 
 		workers_amount = min(32, os.cpu_count() + 4)
@@ -86,7 +94,9 @@ class Bot(object):
 		self.client.add_handler(MessageHandler(self.handlers.help, filters.command("help")))
 		self.client.add_handler(MessageHandler(self.handlers.random, filters.command("random")))
 		self.client.add_handler(MessageHandler(self.handlers.handler))
-		self.client.add_handler(CallbackQueryHandler(self.handlers.simple_button_handler))
+		#TODO refactor to callback router
+		self.client.add_handler(CallbackQueryHandler(self.handlers.simple_button_handler, filters=filters.create(lambda _, __, q: not q.data.startswith("read_more:"))))
+		self.client.add_handler(CallbackQueryHandler(self.handlers.read_more_handler, filters=filters.create(lambda _, __, q: q.data.startswith("read_more:"))))
 
 		self.placeholder = PlaceholderMessage(self)
 
@@ -100,7 +110,7 @@ class Bot(object):
 		self.client.run()
 
 	def stop(self) -> None:
-		logging.info("Warp Beacon terminating. This may take a while ...")
+		logging.info("Warp Beacon is terminating. This may take a while ...")
 		self.scheduler.stop()
 		self.downloader.stop_all()
 		self.uploader.stop_all()
@@ -154,13 +164,30 @@ class Bot(object):
 
 	def build_signature_caption(self, job: UploadJob) -> str:
 		caption = ""
+		is_group = job.chat_type in (ChatType.GROUP, ChatType.SUPERGROUP)
+		is_youtube = job.job_origin in (Origin.YOUTUBE, Origin.YT_SHORTS)
 		if job.canonical_name:
-			caption = f"<b>{html.escape(job.canonical_name)}</b>"
-		if job.chat_type in (ChatType.GROUP, ChatType.SUPERGROUP):
+			if is_group and not is_youtube and CaptionShortner.need_short(job.canonical_name):
+				caption = f"{html.escape(CaptionShortner.smart_truncate_html(job.canonical_name))} ..."
+				job.short_text = True
+			elif is_group:
+				# short enough
+				caption = html.escape(job.canonical_name)
+			# Captions only for YouTube in groups; empty otherwise.
+			# Might be too long for Telegram, so we skip them.
+			if is_youtube and is_group:
+				caption = f"<b>{html.escape(job.canonical_name)}</b>"
+		if is_group:
 			if job.canonical_name:
 				caption += "\n—\n"
+			if job.message_leftover:
+				caption += job.message_leftover
+				caption += "\n—\n"
 			if job.source_username:
-				caption += f"Requested by <b>@{job.source_username}</b>"
+				if job.source_username.startswith("<a href="):
+					caption += f"Requested by <b>{job.source_username}</b>"
+				else:
+					caption += f"Requested by <b>@{job.source_username}</b>"
 			if job.source_username and job.url:
 				caption += " | "
 			if job.url:
@@ -247,7 +274,6 @@ class Bot(object):
 					args["duration"] = round(job.media_info["duration"])
 					args["title"] = job.canonical_name
 					args["caption"] = self.build_signature_caption(job)
-				#args["file_name"] = "%s%s" % (job.canonical_name, os.path.splitext(job.local_media_path)[-1]),
 		elif job.media_type == JobType.ANIMATION:
 			if job.tg_file_id:
 				if job.placeholder_message_id:
@@ -326,10 +352,18 @@ class Bot(object):
 		else:
 			args["disable_notification"] = True
 			args["reply_to_message_id"] = job.message_id
-			#args["reply_to_message_id"] = None
 
-		if os.environ.get("ENABLE_DONATES", None) == "true" and job.media_type is not JobType.COLLECTION:
-			args["reply_markup"] = InlineKeyboardMarkup([[InlineKeyboardButton("❤ Donate", url=os.environ.get("DONATE_LINK", "https://pay.cryptocloud.plus/pos/W5BMtNQt5bJFoW2E"))]])
+		if job.media_type is not JobType.COLLECTION:
+			render_donates = os.environ.get("ENABLE_DONATES", None) == "true"
+			keyboard_buttons = [[]]
+			# short_text is True only for groups
+			if job.short_text:
+				keyboard_buttons[0].append(InlineKeyboardButton("📖 Read more", callback_data=f"read_more:{job.job_origin.value}:{job.uniq_id}"))
+			if render_donates:
+				keyboard_buttons[0].append(InlineKeyboardButton("❤ Donate", url=os.environ.get("DONATE_LINK", "https://pay.cryptocloud.plus/pos/W5BMtNQt5bJFoW2E")))
+
+			if keyboard_buttons[0]:  #job.short_text or render_donates:
+				args["reply_markup"] = InlineKeyboardMarkup(keyboard_buttons)
 
 		return args
 
@@ -393,6 +427,9 @@ class Bot(object):
 							elif msg.photo:
 								tg_file_ids.append(msg.photo.file_id + ':image')
 					logging.info("Uploaded to Telegram")
+					if job.chat_type not in (ChatType.GROUP, ChatType.SUPERGROUP):
+						if job.canonical_name:
+							await self.send_text(chat_id=job.user_id, text=job.canonical_name)
 					break
 				except MultiMediaTooLong as e:
 					logging.error("Failed to upload due telegram limitations :(")
