@@ -11,6 +11,7 @@ from warp_beacon.jobs.download_job import DownloadJob
 from warp_beacon.jobs.upload_job import UploadJob
 from warp_beacon.jobs import Origin
 from warp_beacon.jobs.types import JobType
+from warp_beacon.scraper.link_resolver import LinkResolver
 
 import logging
 
@@ -37,9 +38,12 @@ class Handlers(object):
 			UploadJob(
 				tg_file_id=d["tg_file_id"],
 				chat_id=message.chat.id,
+				user_id=message.from_user.id,
 				media_type=JobType[d["media_type"].upper()],
 				message_id=message.id,
 				chat_type=message.chat.type,
+				uniq_id=d["uniq_id"],
+				job_origin=Origin(d["origin"]),
 				source_username=Utils.extract_message_author(message)
 			)
 		)
@@ -58,6 +62,73 @@ class Handlers(object):
 			"Currently supported: Instagram, YouTube Shorts and YouTube Music."
 		)
 
+	async def upload_wrapper(self, job: UploadJob) -> None:
+		try:
+			if job.replay:
+				logging.info("Replaying job with URL: '%s'", job.url)
+				return await self.queue_job(job.to_download_job(replay=False))
+
+			if job.job_failed and job.job_failed_msg:
+				if job.placeholder_message_id:
+					await self.bot.placeholder.remove(job.chat_id, job.placeholder_message_id)
+					return await self.bot.send_text(chat_id=job.chat_id, text=job.job_failed_msg, reply_id=job.message_id)
+								
+			if job.job_warning and job.job_warning_msg:
+				return await self.bot.placeholder.update_text(job.chat_id, job.placeholder_message_id, job.job_warning_msg)
+								
+			tg_file_ids = await self.bot.upload_job(job)
+			if tg_file_ids:
+				if job.media_type == JobType.COLLECTION and job.save_items:
+					for chunk in job.media_collection:
+						for i in chunk:
+							self.storage.add_media(
+								tg_file_ids=[i.tg_file_id],
+								media_url=i.effective_url,
+								media_type=i.media_type.value,
+								origin=job.job_origin.value,
+								canonical_name=job.canonical_name
+							)
+				else:
+					self.storage.add_media(
+						tg_file_ids=[','.join(tg_file_ids)],
+						media_url=job.url,
+						media_type=job.media_type.value,
+						origin=job.job_origin.value,
+						canonical_name=job.canonical_name
+					)
+		except Exception as e:
+			logging.error("Exception occurred while performing upload callback!")
+			logging.exception(e)
+
+	async def queue_job(self, job: DownloadJob) -> bool:
+		try:
+			# create placeholder message for long download
+			if not job.placeholder_message_id:
+				job.placeholder_message_id = await self.bot.placeholder.create(
+					chat_id=job.chat_id,
+					reply_id=job.message_id
+				)
+
+			if not job.placeholder_message_id:
+				return await self.bot.send_text(
+					chat_id=job.chat_id,
+					reply_id=job.message_id,
+					text="Failed to create message placeholder. Please check your bot Internet connection."
+				)
+
+			self.bot.uploader.add_callback(
+				job.placeholder_message_id,
+				self.upload_wrapper
+			)
+
+			self.bot.downloader.queue_task(job)
+		except Exception as e:
+			logging.error("Failed to schedule download task!")
+			logging.exception(e)
+			return False
+		
+		return True
+
 	async def handler(self, client: Client, message: Message) -> None:
 		if message is None:
 			return
@@ -66,7 +137,13 @@ class Handlers(object):
 			return
 		chat = message.chat
 		effective_message_id = message.id
-		urls = self.url_extractor.find_urls(message_text)
+		urls_raw = self.url_extractor.find_urls(message_text)
+		urls, msg_leftover = [], ''
+		if urls_raw:
+			msg_leftover = Utils.compute_leftover(urls_raw, message_text)
+			msg_leftover = await Utils.handle_mentions(chat.id, client, msg_leftover)
+			# remove duplicates
+			urls = list(set(urls_raw))
 
 		reply_text = "Wut?"
 		if not urls:
@@ -75,10 +152,10 @@ class Handlers(object):
 			for url in urls:
 				origin = Utils.extract_origin(url)
 				if origin is Origin.YOUTU_BE:
-					url = Utils.extract_youtu_be_link(url)
-					if not url:
-						raise ValueError("Failed to extract youtu.be link")
-					origin = Origin.YOUTUBE
+					new_url = LinkResolver.extract_youtu_be_link_local(url)
+					if new_url:
+						url = new_url
+						origin = Origin.YOUTUBE
 				if origin is Origin.UNKNOWN:
 					logging.info("Only Instagram, YouTube Shorts and YouTube Music are now supported. Skipping.")
 					continue
@@ -97,12 +174,16 @@ class Handlers(object):
 						await self.bot.upload_job(
 							UploadJob(
 								url=url,
+								uniq_id=uniq_id,
+								job_origin=origin,
 								tg_file_id=",".join(tg_file_ids),
 								message_id=effective_message_id,
 								media_type=JobType.COLLECTION,
 								chat_id=chat.id,
+								user_id=message.from_user.id,
 								chat_type=message.chat.type,
-								source_username=Utils.extract_message_author(message)
+								source_username=Utils.extract_message_author(message),
+								message_leftover=msg_leftover
 							)
 						)
 					elif ent_len:
@@ -111,86 +192,38 @@ class Handlers(object):
 						await self.bot.upload_job(
 							UploadJob(
 								url=url,
+								uniq_id=uniq_id,
+								job_origin=origin,
 								tg_file_id=tg_file_ids.pop(),
 								message_id=effective_message_id,
 								media_type=media_type,
 								chat_id=chat.id,
+								user_id=message.from_user.id,
 								chat_type=message.chat.type,
 								source_username=Utils.extract_message_author(message),
-								canonical_name=canonical_name
+								canonical_name=canonical_name,
+								message_leftover=msg_leftover
 							)
 						)
 				else:
-					async def upload_wrapper(job: UploadJob) -> None:
-						try:
-							if job.job_failed and job.job_failed_msg:
-								if job.placeholder_message_id:
-									await self.bot.placeholder.remove(chat.id, job.placeholder_message_id)
-								return await self.bot.send_text(chat_id=chat.id, text=job.job_failed_msg, reply_id=job.message_id)
-							if job.job_warning and job.job_warning_msg:
-								return await self.bot.placeholder.update_text(chat.id, job.placeholder_message_id, job.job_warning_msg)
-							tg_file_ids = await self.bot.upload_job(job)
-							if tg_file_ids:
-								if job.media_type == JobType.COLLECTION and job.save_items:
-									for chunk in job.media_collection:
-										for i in chunk:
-											self.storage.add_media(
-												tg_file_ids=[i.tg_file_id],
-												media_url=i.effective_url,
-												media_type=i.media_type.value,
-												origin=job.job_origin.value,
-												canonical_name=job.canonical_name
-											)
-								else:
-									self.storage.add_media(
-										tg_file_ids=[','.join(tg_file_ids)],
-										media_url=job.url,
-										media_type=job.media_type.value,
-										origin=job.job_origin.value,
-										canonical_name=job.canonical_name
-									)
-						except Exception as e:
-							logging.error("Exception occurred while performing upload callback!")
-							logging.exception(e)
-
-					try:
-						# create placeholder message for long download
-						placeholder_message_id = await self.bot.placeholder.create(
-							chat_id=chat.id,
-							reply_id=effective_message_id
-						)
-
-						if not placeholder_message_id:
-							await self.bot.send_text(
-								chat_id=chat.id,
-								reply_id=effective_message_id,
-								text="Failed to create message placeholder. Please check your bot Internet connection.")
-							return
-
-						self.bot.uploader.add_callback(
-							placeholder_message_id,
-							upload_wrapper
-						)
-
-						self.bot.downloader.queue_task(DownloadJob.build(
+					if await self.queue_job(DownloadJob.build(
 							url=url,
-							placeholder_message_id=placeholder_message_id,
 							message_id=effective_message_id,
 							chat_id=chat.id,
+							user_id=message.from_user.id,
 							in_process=self.bot.uploader.is_inprocess(uniq_id),
 							uniq_id=uniq_id,
 							job_origin=origin,
 							source_username=Utils.extract_message_author(message),
-							chat_type=chat.type
-						))
+							chat_type=chat.type,
+							message_leftover=msg_leftover
+						)):
 						self.bot.uploader.set_inprocess(uniq_id)
-					except Exception as e:
-						logging.error("Failed to schedule download task!")
-						logging.exception(e)
 
 		if chat.type not in (ChatType.GROUP, ChatType.SUPERGROUP) and not urls:
 			await self.bot.send_text(text=reply_text, reply_id=effective_message_id, chat_id=chat.id)
 
+	#TODO refactor to callback router
 	async def simple_button_handler(self, client: Client, query: CallbackQuery) -> None:
 		await client.answer_callback_query(
 			callback_query_id=query.id,
@@ -198,4 +231,49 @@ class Handlers(object):
 			show_alert=True
 		)
 		self.bot.downloader.auth_event.set()
-		self.bot.downloader.auth_event.clear()
\ No newline at end of file
+		self.bot.downloader.auth_event.clear()
+
+	async def read_more_handler(self, client: Client, query: CallbackQuery) -> None:
+		origin, uniq_id = '', ''
+		#read_more:{job.job_origin}:{job.uniq_id}
+		if query.data:
+			parts = query.data.split(':')
+			if len(parts) == 3:
+				_, origin, uniq_id = parts
+		logging.info("Handling read_more request: uniq_id='%s', origin='%s'", uniq_id, origin)
+		db_results = []
+		if uniq_id and origin:
+			db_results = self.storage.db_find(uniq_id=uniq_id.strip(), origin=origin.strip())
+		first_entity = {}
+		if db_results:
+			first_entity = db_results[0]
+		
+		text = first_entity.get("canonical_name", "Failed to fetch data.")
+
+		try:
+			await client.send_message(
+				query.from_user.id,
+				text,
+				parse_mode=ParseMode.HTML
+			)
+		except Exception as _:
+			try:
+				await client.answer_callback_query(
+					callback_query_id=query.id,
+					show_alert=True,
+					text="You haven’t messaged the bot yet. Please start the chat using the /start bot command."
+				)
+			except Exception as e:
+				logging.warning("Failed to return error to user about TG restrictions!")
+				logging.exception(e)
+			return
+
+		try:
+			await client.answer_callback_query(
+				callback_query_id=query.id,
+				show_alert=True,
+				text="Check your private chat — I've sent you the message!"
+			)
+		except Exception as e:
+			logging.error("Failed to report sent message status!")
+			logging.exception(e)
